package edu.nyu.scps.sqlite;

import android.app.AlertDialog;
import android.content.ContentValues;
import android.content.DialogInterface;
import android.database.Cursor;
import android.database.sqlite.SQLiteDatabase;
import android.os.AsyncTask;
import android.os.Bundle;
import android.support.v7.app.AppCompatActivity;
import android.text.Editable;
import android.util.Log;
import android.view.KeyEvent;
import android.view.LayoutInflater;
import android.view.Menu;
import android.view.MenuItem;
import android.view.View;
import android.widget.AdapterView;
import android.widget.EditText;
import android.widget.ListView;
import android.widget.SimpleCursorAdapter;
import android.widget.TextView;

public class MainActivity extends AppCompatActivity {
    private String databaseName = "stooges.db";
    private String tableName = "people";
    private String[] columns = {"_id", "name"};
    private Helper helper;   //Can't initialize this field before onCreate.
    private SQLiteDatabase db;
    private SimpleCursorAdapter adapter;

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_main);

        final ListView listView = (ListView) findViewById(R.id.listView);
        TextView textView = (TextView)findViewById(R.id.empty);
        listView.setEmptyView(textView);   //Display this TextView when table contains no records.

        adapter = new SimpleCursorAdapter(
                this,
                android.R.layout.simple_list_item_2,
                null,
                new String[] {"name",             "_id"},
                new int[]    {android.R.id.text1, android.R.id.text2},
                0    //don't need any flags
        );
        listView.setAdapter(adapter);

        helper = new Helper(this, databaseName);
        CreateTask createTask = new CreateTask();
        createTask.execute();

        listView.setOnItemClickListener(new AdapterView.OnItemClickListener() {
            @Override
            public void onItemClick(final AdapterView<?> parent, View view, final int position, final long id) {
                AlertDialog.Builder builder = new AlertDialog.Builder(MainActivity.this);
                builder.setTitle(R.string.title_update_or_delete);
                Cursor cursor = (Cursor)parent.getItemAtPosition(position); //downcast
                int _idIndex = cursor.getColumnIndex("_id");
                int nameIndex = cursor.getColumnIndex("name");
                builder.setMessage(String.format(getString(R.string.message_update), cursor.getLong(_idIndex)));

                LayoutInflater inflater = getLayoutInflater();
                View v = inflater.inflate(R.layout.edit_text, null);
                builder.setView(v);

                builder.setNeutralButton(R.string.button_cancel, null); //Make dialog disappear without doing anything.

                builder.setNegativeButton(R.string.button_delete, new DialogInterface.OnClickListener() {
                    @Override
                    public void onClick(DialogInterface dialog, int which) {
                        db.delete(tableName, "_id = ?", new String[] {Long.toString(id)});
                        Cursor cursor = db.query(tableName, columns, null, null, null, null, null);
                        adapter.changeCursor(cursor);
                    }
                });

                final AlertDialog alertDialog = builder.create();
                EditText editText = (EditText)v.findViewById(R.id.editText);
                editText.setText(cursor.getString(nameIndex));
                editText.setHint(R.string.hint_update);
                editText.setImeActionLabel(getString(R.string.key_update), 0);

                editText.setOnKeyListener(new View.OnKeyListener() {
                    @Override
                    public boolean onKey(View v, int keyCode, KeyEvent event) {
                        if (event.getAction() != KeyEvent.ACTION_DOWN || keyCode != KeyEvent.KEYCODE_ENTER) {
                            return false;
                        }
                        if (db != null) {
                            EditText editText = (EditText)v;
                            Editable editable = editText.getText();
                            ContentValues contentValues = new ContentValues();
                            contentValues.put("name", editable.toString());
                            db.update(tableName, contentValues, "_id = ?", new String[] {Long.toString(id)});
                            Cursor cursor = db.query(tableName, columns, null, null, null, null, null);
                            adapter.changeCursor(cursor);
                        }
                        alertDialog.dismiss();
                        return true;
                    }
                });
                alertDialog.show();
            }
        });
    }

    private class CreateTask extends AsyncTask<Void, Void, SQLiteDatabase> {
        @Override
        protected SQLiteDatabase doInBackground(Void... params) {
            return helper.getWritableDatabase();
        }

        @Override
        protected void onPostExecute(SQLiteDatabase db) {
            MainActivity.this.db = db;
            Cursor cursor = db.query(tableName, columns, null, null, null, null, null);
            adapter.swapCursor(cursor);
        }
    }

    @Override
    public boolean onCreateOptionsMenu(Menu menu) {
        // Inflate the menu; this adds items to the action bar if it is present.
        getMenuInflater().inflate(R.menu.menu_main, menu);
        return true;
    }

    @Override
    public boolean onOptionsItemSelected(MenuItem item) {
        // Handle action bar item clicks here. The action bar will
        // automatically handle clicks on the Home/Up button, so long
        // as you specify a parent activity in AndroidManifest.xml.
        int id = item.getItemId();

        //noinspection SimplifiableIfStatement
        if (id == R.id.action_settings) {
            return true;
        }

        if (id == R.id.action_append) {
            AlertDialog.Builder builder = new AlertDialog.Builder(this);
            builder.setTitle(R.string.title_new);
            builder.setMessage(R.string.message_new);

            LayoutInflater inflater = getLayoutInflater();
            View view = inflater.inflate(R.layout.edit_text, null);
            builder.setView(view);

            builder.setNegativeButton(R.string.button_cancel, null);
            final AlertDialog alertDialog = builder.create();
            EditText editText = (EditText) view.findViewById(R.id.editText);
            editText.setHint(R.string.hint_create);
            editText.setImeActionLabel(getString(R.string.key_create), 0);

            editText.setOnKeyListener(new View.OnKeyListener() {
                @Override
                public boolean onKey(View v, int keyCode, KeyEvent event) {
                    if (event.getAction() != KeyEvent.ACTION_DOWN || keyCode != KeyEvent.KEYCODE_ENTER) {
                        return false;
                    }
                    if (db != null) {
                        EditText editText = (EditText) v;
                        Editable editable = editText.getText();
                        ContentValues contentValues = new ContentValues();
                        contentValues.put("name", editable.toString());
                        db.insert(tableName, null, contentValues);
                        Cursor cursor = db.query(tableName, columns, null, null, null, null, null);
                        adapter.changeCursor(cursor);
                    }
                    alertDialog.dismiss();
                    return true;
                }
            });

            alertDialog.show();
            return true;
        }

        if (id == R.id.action_delete_all) {
            if (db != null) {
                //Delete all the records, but do not destroy the database or the table.
                db.delete(tableName, null, null);
                Cursor cursor = db.query(tableName, columns, null, null, null, null, null);
                adapter.changeCursor(cursor);
            }
            return true;
        }

        if (id == R.id.action_reset) {
            //Destroy the database and re-create it.
            adapter.changeCursor(null); //close the previous cursor
            db = null;
            helper.close();
            if (!deleteDatabase(databaseName)) {
                Log.e("myTag", "delete database");
            }
            helper = new Helper(this, databaseName);
            CreateTask createTask = new CreateTask();
            createTask.execute();
            return true;
        }

        return super.onOptionsItemSelected(item);
    }
}
